#!/usr/bin/python
# -*- coding: utf-8 -*-

import kinterbasdb
import urllib
from lxml.html import parse, HTMLParser
from itertools import izip_longest
from threading import Thread, Lock
import time
from datetime import datetime
import argparse
from configobj import ConfigObj
import os

class fb:
    "Робота з FB"
    def __init__(self, section):
        self.con = kinterbasdb.connect(
            host = fb_config[section]['host'], database = fb_config[section]['name'],
            user = fb_config[section]['user'], password = fb_config[section]['pass'], charset = fb_config[section]['charset']
        )
 
    def selectSQL(self, query, params=([])):
        "Select data from db"
        cur = self.con.cursor()
        cur.set_type_trans_in ({'BLOB': {'mode': 'stream'}})
        cur.set_type_trans_out({'BLOB': {'mode': 'stream'}})
        try:
            cur.execute(query, params)
            result = cur.fetchall()
            cur.close()
            self.con.commit()
        except kinterbasdb.ProgrammingError (errNumb, errText):
            cur.close()
            self.con.rollback()
            result = (errNumb, errText, query)
        except:
            cur.close()
            self.con.rollback()
            result = (-1, 'Unknown error', query)
        finally:
            return result

    def execSQL(self, query, params=([])):
        cur = self.con.cursor()
        cur.set_type_trans_in ({'BLOB': {'mode': 'stream'}})
        cur.set_type_trans_out({'BLOB': {'mode': 'stream'}})
        try:
            cur.execute(query, params)
        except:
            cur.close()
            self.con.rollback()
            result = (-1, 'Unknown error', query)
        else:
            cur.close()
            self.con.commit()
            result = (0, 'Success', query)
        finally:
            return result
# кінець класу для роботи з базою

def processYarnart(pageurl):
    fbclient = fb('yarn')
    site_id = 'yarnart.info'
    manufacturer = 'YarnArt'
    report(datetime.now().strftime("%d.%m.%Y %X") + ' початок парсингу yarnart.info\n')
    page = parse(pageurl).getroot()
    for href in page.cssselect("option"):
        if int(href.get("value"))>0:
            source_url = "http://www.yarnart.info/UrunDetay.aspx?uid=" + href.get("value") + "&lang=en-US"
            external_id = int(source_url[43:-11])
            if fbclient.selectSQL('select 1 from yarns where site_id = ? and external_id = ?', (site_id, external_id)) == []:
                doc = parse(source_url).getroot()
                series = doc.cssselect('span#ctl00_ContentPlaceHolder1_lblUrunAdi')[0].text
                content = doc.cssselect('span#ctl00_ContentPlaceHolder1_lblUrunAciklama')[0].text
                weight = doc.cssselect('span#ctl00_ContentPlaceHolder1_lblYumakAgirlikAciklama')[0].text
                length = doc.cssselect('span#ctl00_ContentPlaceHolder1_lblTahminiUzunlukAciklama')[0].text
                using = doc.cssselect('span#ctl00_ContentPlaceHolder1_lblTigSisOlcu')[0].text
                photo = "http://www.yarnart.info/" + doc.cssselect('img#ctl00_ContentPlaceHolder1_imgYumakDetay')[0].get('src')
                colorcodes =[i.strip() for i in doc.xpath('//table[@id="ctl00_ContentPlaceHolder1_DataList1"]/tr/td/ul/li/p/text()')]
                color_imgs = ['http://www.yarnart.info/' + i for i in doc.xpath('//table[@id="ctl00_ContentPlaceHolder1_DataList1"]/tr/td/ul/li/img/@src')]
                color_group = zip(colorcodes, color_imgs)
                yarnToBase(manufacturer, series, content, length, weight, using, photo, site_id, source_url, 0, external_id, color_group, fbclient)
    report(datetime.now().strftime("%d.%m.%Y %X") + ' кінець парсингу yarnart.info\n')
        
def processAlize(pageurl):
    fbclient = fb('yarn')
    site_id = 'alize.gen.tr'
    manufacturer = 'Alize'
    report(datetime.now().strftime("%d.%m.%Y %X") + ' початок парсингу alize.gen.tr\n')
    page = parse(pageurl).getroot()
    for link in page.cssselect('span.yumak_yazi a'):
        source_url = 'http://www.alize.gen.tr/index_en.php' + link.get('href')
        external_id = int(source_url[60:])
        if fbclient.selectSQL('select 1 from yarns where site_id = ? and external_id = ?', (site_id, external_id)) == []:
            doc = parse(source_url).getroot()
            series = doc.cssselect('td.urun_adi div')[0].text.strip()
            content = doc.cssselect('div.urun_yazi')[0].text.strip()
            length = doc.xpath('//td[@class="urun_yazi"]/div/b/following-sibling::text()[1]')[0].split()[2]
            package = doc.xpath('//td[@class="urun_yazi"]/div/b/following-sibling::text()[1]')[2].strip()
            using = '-'.join(doc.xpath('//table[@width="118"]/tr/td/text()')).strip().replace('%s' % u'\xa0\xa0', ' ')
            photo = 'http://www.alize.gen.tr/' + doc.cssselect('a.cloud-zoom')[0].get('href')
            xpath1 = doc.cssselect('table')[5]
            colorcodes = xpath1.xpath('tr/td/div/div[@class="renk_no3"]/div/text()')
            if colorcodes == []:
                colorcodes = xpath1.xpath('tr/td/div/div/text()')
            color_names = xpath1.xpath('tr/td/div/div/div/table/tr/td/text()')
            color_imgs = ['http://www.alize.gen.tr/' + i for i in xpath1.xpath('tr/td/a/img/@src')]
            color_group = izip_longest(colorcodes, color_names, color_imgs)
            qu = fbclient.selectSQL('''insert into yarns(manufacturer, series, contents,
                length, weight, use_method, photo, site_id, source_url, parsing_result, external_id) values(?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
                returning id''', 
                (manufacturer, series, content, length, package, using, urllib.urlopen(photo), site_id, source_url, 0, external_id))
            if qu[0] == -1:
                print 'Error processing category %s' % series
            #print 'Adding colors for %s' % series
            yarn_id = qu[0]
            for colorcode, color_name, color_img in color_group:
                qu = fbclient.execSQL('insert into yarncolors(yarn_id, color_id, color_name, photo)values(?,?,?,?)',
                                      (yarn_id[0], colorcode, color_name, urllib.urlopen(color_img)))
                #if qu[0] != 0:
                #    print "%s happened during executing %s" %(qu[1], qu[2])
    report(datetime.now().strftime("%d.%m.%Y %X") + ' кінець парсингу alize.gen.tr\n')

def processKartopu(pageurl):
    fbclient = fb('yarn')
    site_id = 'kartopu.com.tr'
    manufacturer = 'Kartopu'
    report(datetime.now().strftime("%d.%m.%Y %X") + ' початок парсингу kartopu.com.tr\n')
    page = parse(pageurl, HTMLParser(encoding="utf-8"))
    series = page.xpath('//div[@class="product1"]/text()')
    weights = page.xpath('//div[@class="product2"]/text()')
    lengths = page.xpath('//div[@class="product3"]/text()')
    links = ['http://www.kartopu.com.tr' + link for link in page.xpath('//a[@class="detay"]/@href')]
    groups = zip(series, weights, lengths, links)
    for series, weight, length, source_url in groups:
        external_id = int(source_url[32:-1])
        if fbclient.selectSQL('select 1 from yarns where site_id = ? and external_id = ?', (site_id, external_id)) == []:
            doc = parse(source_url, HTMLParser(encoding="utf-8"))
            content = doc.xpath('//td[@class="productDetailsHedar"]/text()')[0].strip()
            photo = doc.xpath('//img[@id="ctl00_ctl00_ContentPlaceHolder1_ContentPlaceHolder2_imgProduct"]/@src')[0]
            colorcodes = doc.xpath('//span[@class="productColorName2"]/text()')
            color_imgs = [color_img.split("'")[1].encode('utf-8') for color_img in doc.xpath('//div[@class="productColorImages"]/@style')]
            color_group = izip_longest(colorcodes, color_imgs)
            yarnToBase(manufacturer, series, content, length, weight, None, photo, site_id, source_url, 0, external_id, color_group, fbclient)
    report(datetime.now().strftime("%d.%m.%Y %X") + ' кінець парсингу kartopu.com.tr\n')
        
def processGazzal(pageurl):
    fbclient = fb('yarn')
    site_id = 'gazzal.net'
    manufacturer = 'Gazzal'
    report(datetime.now().strftime("%d.%m.%Y %X") + ' початок парсингу gazzal.net\n')
    page = parse(pageurl).getroot()
    for link in page.cssselect('div.yarns_urun_baslik a'):
        source_url = 'http://www.gazzal.net/' + link.get('href')
        external_id = int(source_url[50:-6])
        if fbclient.selectSQL('select 1 from yarns where site_id = ? and external_id = ?', (site_id, external_id)) == []:
            doc = parse(source_url).getroot()
            series = doc.cssselect('div.baslik')[0].text
            content = doc.cssselect('td.tablo_sola_yasla')[0].text
            content = ''.join(content.splitlines())
            weight = doc.cssselect('td.tablo_sola_yasla')[1].text.replace("/", " ").split()[2]
            length = doc.cssselect('td.tablo_sola_yasla')[1].text.split()[0]
            photo = 'http://www.gazzal.net/' + doc.cssselect('a#example1')[0].get('href')
            colorcodes = doc.xpath('//div[@class="colour_s"]/text()')
            color_imgs = ['http://www.gazzal.net/' + i for i in doc.xpath('//div[@class="colour_box"]/a/@href')]
            color_group = zip(colorcodes, color_imgs)
            yarnToBase(manufacturer, series, content, length, weight, None, photo, site_id, source_url, 0, external_id, color_group, fbclient)
    report(datetime.now().strftime("%d.%m.%Y %X") + ' кінець парсингу gazzal.net\n')
                
def processVizell(pageurl):
    fbclient = fb('yarn')
    site_id = 'vizell.com.ua'
    manufacturer = 'Vizell'
    report(datetime.now().strftime("%d.%m.%Y %X") + ' початок парсингу vizell.com.ua\n')
    page = parse(pageurl).getroot()
    for link in page.cssselect('table tbody tr td p a'):
        source_url = link.get('href')
        if fbclient.selectSQL('select 1 from yarns where source_url = ?', (source_url,)) == []:
            doc = parse(source_url, HTMLParser(encoding="utf-8"))
            series = doc.xpath('//h1')[0].text
            try:
                content = doc.xpath('//font/b/following-sibling::text()[1]')[0]
                photo = doc.xpath('//div[@class="one_third"]/div/a')[0].get('href')
                try:
                    weight = doc.xpath('//font/b/following-sibling::text()[1]')[1].split()[3]
                    length = doc.xpath('//font/b/following-sibling::text()[1]')[1].split()[0]
                except: 
                    weight = doc.xpath('//font/b/following-sibling::text()[1]')[2].split()[2]
                    length = doc.xpath('//font/b/following-sibling::text()[1]')[2].split()[0]
            except:
                content = doc.xpath('//span/strong/following-sibling::text()[1]')[0]
                weight = doc.xpath('//span/strong/following-sibling::text()[1]')[1].split()[3]
                length = doc.xpath('//span/strong/following-sibling::text()[1]')[1].split()[0]
                photo = doc.xpath('//p/a/@href')[0]
            colorcodes = doc.xpath('//div[@class="two_third last"]/div/a/@title')
            color_imgs = doc.xpath('//div[@class="two_third last"]/div/a/@href')
            color_group = izip_longest(colorcodes, color_imgs)
            yarnToBase(manufacturer, series, content, length, weight, None, photo, site_id, source_url, 0, None, color_group, fbclient)
            time.sleep(1)
    report(datetime.now().strftime("%d.%m.%Y %X") + ' кінець парсингу vizell.com.ua\n')

def processPeriayarn(pageurl):
    fbclient = fb('yarn')
    site_id = 'periayarn.com.ua'
    manufacturer = 'Peria'
    report(datetime.now().strftime("%d.%m.%Y %X") + ' початок парсингу periayarn.com.ua\n')
    page = parse(pageurl).getroot()
    for link in page.cssselect('div.plist table tr td a'):
        source_url = 'http://www.periayarn.com.ua/products.php' + link.get('href')
        external_id = int(source_url[45:])
        if fbclient.selectSQL('select 1 from yarns where site_id = ? and external_id = ?', (site_id, external_id)) == []:
            doc = parse(source_url).getroot()
            series = doc.cssselect('form#NewRecord2 table tr td strong')[0].text
            content = doc.cssselect('form#NewRecord2 table tr td')[2].text
            weight = doc.cssselect('form#NewRecord2 table tr td')[6].text
            length = doc.cssselect('form#NewRecord2 table tr td')[4].text
            photo = 'http://www.periayarn.com.ua/' + doc.cssselect('form#NewRecord1 table tr td img')[0].get('src')
            colorcodes = doc.cssselect('table')[4].xpath('tr/td/input[1]/@value')
            color_imgs = ['http://www.periayarn.com.ua/' + i for i in doc.xpath('//img[@height="20"]/@src')]
            color_group = zip(colorcodes, color_imgs)
            yarnToBase(manufacturer, series, content, length, weight, None, photo, site_id, source_url, 0, external_id, color_group, fbclient)
    report(datetime.now().strftime("%d.%m.%Y %X") + ' кінець парсингу periayarn.com.ua\n')
        
def processZnannia(pageurl):
    fbclient = fb('book')
    site_id = 'znannia.com.ua'
    report(datetime.now().strftime("%d.%m.%Y %X") + ' початок парсингу znannia.com.ua\n')
    page = parse(pageurl).getroot()
    sections = page.xpath('//a[@class="sublevel"]/@href')
    numbers = [int(i.split('(')[1][:-2]) for i in page.xpath('//a[@class="sublevel"]/text()')]
    sections_group = zip(sections, numbers)
    for section, number in sections_group:
        if number != 0:
            for num in range(0, number, 10):
                books = parse(section + '&limit=10&limitstart=' + str(num)).getroot()
                for source_url in books.xpath('//a[@title="%s"]/@href' % u'\u0414\u0435\u0442\u0430\u043b\u044c\u043d\u0456\u0448\u0435'):
                    external_id = int(source_url.split('product_id=')[1].split('&')[0])
                    if fbclient.selectSQL('select 1 from books where site_id = ? and external_id = ?', (site_id, external_id)) == []:
                        try:
                            book = parse(source_url).getroot()
                            full_name = book.cssselect('h2')[0].text
                            #print full_name, external_id
                            try:
                                author = book.cssselect('strong')[1].text
                                if len(author) > 50:
                                    author = book.cssselect('strong')[1].text.split('/')[1].split(u'\u2014')[0].strip()
                            except:
                                author = None
                            try:
                                isbn = ' '.join(book.cssselect('meta')[0].get('content').splitlines()).split('ISBN')[1].strip()
                            except:
                                isbn = None
                            try:
                                pages = int(book.cssselect('meta')[0].get('content').split(u'\u2014')[-1].split()[0])
                            except:
                                pages = None
                            photo = book.xpath('//a[@target="_blank"]/@href')[0]
                            descr = ' '.join(book.cssselect('meta')[0].get('content').splitlines())[63:]
                            bookToBase(full_name, author, isbn, None, None, None, pages, None, photo, descr, None, None, site_id, source_url, 0, external_id, fbclient)
                        except:
                            report(source_url + '   <-- Тут обламався парсер\n')
    report(datetime.now().strftime("%d.%m.%Y %X") + ' кінець парсингу znannia.com.ua\n')

def processOzon(pageurl):
    fbclient = fb('book')
    site_id = 'ozon.ru'
    report(datetime.now().strftime("%d.%m.%Y %X") + ' початок парсингу ozon.ru\n')
    i = 83058
    while i<19744075 :
        source_url = pageurl + str(i)
        external_id = int(source_url.split('/')[-1])
        if fbclient.selectSQL('select 1 from books where site_id = ? and external_id = ?', (site_id, external_id)) == [] and (urllib.urlopen(source_url).getcode() in [200, 301]):
            book = parse(source_url).getroot()
            if (book.xpath('//li[@class="first"]/a') != [] ) and (book.xpath('//li[@class="first"]/a/@href')[0] == '/context/div_book/') :
                try:
                    full_name = book.cssselect('h1')[0].text
                    #print full_name, external_id
                    author = book.cssselect('div.product-detail p a')[0].text
                    try:
                        isbn = book.cssselect('title')[0].text.split('/')[1].strip()[:80]
                    except:
                        isbn = None
                    try:
                        publisher = book.xpath('//p[text()="%s"]/a/text()' % u'\u0418\u0437\u0434\u0430\u0442\u0435\u043b\u044c\u0441\u0442\u0432\u043e:\r\n\t\t\t\t')[0]
                    except:
                        publisher = None
                    try:
                        series = book.xpath('//p[text()="%s"]/a/text()' % u'\u0421\u0435\u0440\u0438\u044f: ')[0]
                    except:
                        series = None
                    try:
                        forma = book.xpath('//div[span/text()="%s"]/following-sibling::div/span/text()' % u'\u0424\u043e\u0440\u043c\u0430\u0442')[0]
                    except:
                        forma = None
                    try:
                        pages = int(book.cssselect('div.techDescr span')[0].text.split()[0])
                    except:
                        pages = None
                    cover = book.cssselect('span#byType')[0].text
                    photo = 'http:' + book.cssselect('img#js_article_picture')[0].get('src')
                    descr = reduce( lambda res, x: res+x+' ',  [row.strip() for row in book.xpath('//h3/following-sibling::text()')])
                    bookToBase(full_name, author, isbn, publisher, series, forma, pages, cover, photo, descr, None, None, site_id, source_url, 0, external_id, fbclient)
                except:
                    report(source_url + '   <-- Тут обламався парсер\n')
            time.sleep(2)
        i+= 1
    report(datetime.now().strftime("%d.%m.%Y %X") + ' кінець парсингу ozon.ru\n')
                        
def processKnigka(pageurl):
    fbclient = fb('book')
    site_id = 'knigka.ua'
    report(datetime.now().strftime("%d.%m.%Y %X") + ' початок парсингу knigka.ua\n')
    page = parse(pageurl).getroot()
    for section in page.cssselect('li.col_black a'):
        page = parse('http://knigka.ua' + section.get('href')).getroot()
        if page.cssselect('div.modern-page-navigation a') == []:
            last = 1
        else:
            last = int(page.cssselect('div.modern-page-navigation a')[-2].text)
        for link in ['http://knigka.ua' + section.get('href') + 'PAGEN_5-' + str(i) + '/' for i in range(1, last+1)]:
            books = parse(link).getroot()
            for onebook in books.xpath('//table[@class="cat_img_box"]/tr/td/a/@href'):
                source_url = 'http://knigka.ua' + onebook
                external_id = int(source_url.split('/')[-2])
                if fbclient.selectSQL('select 1 from books where site_id = ? and external_id = ?', (site_id, external_id)) == []:
                    try:
	                book = parse(source_url).getroot()
	                full_name = book.cssselect('h1')[0].text
	                #print full_name
	                try:
	                    author = book.xpath('//b[text()="%s"]/following-sibling::a/text()' % u'\u0410\u0432\u0442\u043e\u0440:')[0]
	                except:
                            author = None
                        isbn = book.xpath('//b[text()="ISBN:"]/following-sibling::span/text()')[0]
                        try:
                            publisher = book.xpath('//div[@class="margb"]/a[following-sibling::text()]/text()')[0]
                        except:
                            publisher = None
                        try:
                            series = book.xpath('//b[text()="%s"]/following-sibling::a/text()' % u'\u0421\u0435\u0440\u0438\u044f:')[0]
                        except:
                            series = None
                        forma = book.xpath('//b[text()="%s"]/following-sibling::text()[1]' % u'\u0424\u043e\u0440\u043c\u0430\u0442:')[0].strip()
                        try:
                           pages = int(book.xpath('//div[@class="margb"]/a/following-sibling::text()[1]')[0][2:-2])
                        except:
                            pages = None
                        cover = book.xpath('//b[text()="%s"]/following-sibling::text()[1]' % u'\u041f\u0435\u0440\u0435\u043f\u043b\u0435\u0442:')[0][1:-2]
                        photo = 'http://knigka.ua' + book.cssselect('td.b_w_g img')[0].get('src')
                        descr = ' '.join(book.xpath('//div[@id="elm_desc"]/br/following-sibling::text()')[0].splitlines())
                        bookToBase(full_name, author, isbn, publisher, series, forma, pages, cover, photo, descr, None, None, site_id, source_url, 0, external_id, fbclient)
                    except:
                        report(source_url + '   <-- Тут обламався парсер\n')
                    time.sleep(2)
    report(datetime.now().strftime("%d.%m.%Y %X") + ' кінець парсингу knigka.ua\n')

def processBao(pageurl):
    fbclient = fb('book')
    site_id = 'bao-book.com'
    report(datetime.now().strftime("%d.%m.%Y %X") + ' початок парсингу bao-book.com\n')
    for source_url in [pageurl + str(i) for i in range(9, 2175)]:
        external_id = int(source_url.split('/')[-1])
        if fbclient.selectSQL('select 1 from books where site_id = ? and external_id = ?', (site_id, external_id)) == []:
            if (urllib.urlopen(source_url).getcode() in [200, 301]):
                try:
	            book = parse(source_url).getroot()
	            full_name = book.cssselect('h1')[0].text
	            #print full_name, external_id
	            author = book.xpath('//b[text()="%s"]/following-sibling::text()' % u'\u0410\u0432\u0442\u043e\u0440\u044b:')[0].strip()
                    isbn = book.xpath('//b[text()="ISBN:"]/following-sibling::text()')[0].strip()
                    forma = book.xpath('//b[text()="%s"]/following-sibling::text()' % u'\u0424\u043e\u0440\u043c\u0430\u0442:')[0].strip()
                    pages = int(book.xpath('//b[text()="%s"]/following-sibling::text()' % u'\u041a\u043e\u043b-\u0432\u043e \u0441\u0442\u0440\u0430\u043d\u0438\u0446:')[0].strip())
                    cover = book.xpath('//b[text()="%s"]/following-sibling::text()' % u'\u041f\u0435\u0440\u0435\u043f\u043b\u0435\u0442:')[0].strip()
                    photo = 'http://bao-book.com' + book.cssselect('center a')[0].get('href')
                    descr = ' '.join(book.cssselect('div.summary')[0].text.splitlines())
                    try:
                        date_publishing = book.xpath('//div[@class="pub info"]/b/following-sibling::text()')[0].strip()
                    except:
                        date_publishing = None
                    bookToBase(full_name, author, isbn, None, None, forma, pages, cover, photo, descr, date_publishing, None, site_id, source_url, 0, external_id, fbclient)
                except:
                    report(source_url + '   <-- Тут обламався парсер\n')
    report(datetime.now().strftime("%d.%m.%Y %X") + ' кінець парсингу bao-book.com\n')

def processSchoolbook(pageurl):
    fbclient = fb('book')
    site_id = 'schoolbook.com.ua'
    report(datetime.now().strftime("%d.%m.%Y %X") + ' початок парсингу schoolbook.com.ua\n')
    for section in [pageurl + '?ddlCategory=' + str(i) for i in [3, 4, 5, 6]]:
        page = parse(section).getroot()
        for source_url in [section + '&pgnPager_page=' + str(i) for i in range(1, int(page.cssselect('a.pagenavigator')[-1].get('href').split('=')[1]) + 1)]:
            books = parse(source_url).getroot()
            for book in books.xpath('//div[div[@class="catalog_box_sub"]]'):
                external_id = book.cssselect('img')[0].get('rel')[1:-6]
                if fbclient.selectSQL('select 1 from books where site_id = ? and external_id = ?', (site_id, external_id)) == []:
                    try:
                        full_name = book.cssselect('h3')[0].text
	                #print full_name, external_id
	                author = book.xpath('div/div[@class="info_book"]/span[text()="%s"]/following-sibling::text()' % u'\u0410\u0432\u0442\u043e\u0440:')[0].strip()
                        try:
                            series = book.xpath('div/div[@class="info_book"]/span[text()="%s"]/following-sibling::text()' % u'\u0421\u0435\u0440\u0456\u044f:')[0].strip()
                        except:
                            series = None
                        forma = ' '.join(book.xpath('div/div[@class="info_book"]/following-sibling::text()')[-1].split(', ')[0].splitlines()).strip()
                        pages = int(book.xpath('div/div[@class="info_book"]/following-sibling::text()')[-1].split(',')[-2].split()[0])
                        cover = book.xpath('div/div[@class="info_book"]/following-sibling::text()')[-1].split(',')[-1].strip()
                        photo = book.xpath('following-sibling::div/div/img//@src')[0].encode('utf-8')
                        descr = reduce( lambda res, x: res+x+' ',  [row.strip() for row in book.xpath('div/div[@class="info_book"]/following-sibling::text()')])
                        bookToBase(full_name, author, None, None, series, forma, pages, cover, photo, descr, None, None, site_id, source_url, 0, external_id, fbclient)
                    except:
                        report(source_url + '   <-- Тут обламався парсер\n')
            time.sleep(1)
    report(datetime.now().strftime("%d.%m.%Y %X") + ' кінець парсингу schoolbook.com.ua\n')

def processEksmo(pageurl):
    fbclient = fb('book')
    site_id = 'eksmo.ru'
    report(datetime.now().strftime("%d.%m.%Y %X") + ' початок парсингу eksmo.ru\n')
    page = parse(pageurl).getroot()
    for section_id in [i.get('rel') for i in page.cssselect('ul.rubMenu li a')]:
        page = parse(pageurl + section_id + '/#nav').getroot()
        for link in [pageurl + section_id + '/?SECTION_ID=' + section_id + '&PAGEN_1=' + str(i) for i in range(1, int(page.cssselect('div.navigation-pages a')[4].text) + 1)]:
            books = parse(link).getroot()
            for onebook in books.cssselect('a.blue'):
                source_url = 'http://eksmo.ru' + onebook.get('href')
                external_id = int(source_url.split('/')[-2])
                if fbclient.selectSQL('select 1 from books where site_id = ? and external_id = ?', (site_id, external_id)) == []:
                    try:
	                book = parse(source_url).getroot()
	                full_name = book.cssselect('h1.blue')[0].text
	                #print full_name, external_id
	                try:
	                    author = book.xpath('//strong[text()="%s"]/following-sibling::span/a/text()' % u'\u0410\u0432\u0442\u043e\u0440:')[0]
	                except:
                            author = None
                        isbn = book.xpath('//strong[text()="ISBNN:"]/following-sibling::span/text()')[0]
                        try:
                            series = book.xpath('//strong[text()="%s"]/following-sibling::span/a/text()' % u'C\u0435\u0440\u0438\u044f:')[0]
                        except:
                            series = None
                        forma = book.xpath('//strong[text()="%s"]/following-sibling::span/text()' % u'\u0424\u043e\u0440\u043c\u0430\u0442:')[0]
                        pages = int(book.xpath('//strong[text()="%s"]/following-sibling::span/text()' %
                                               u'\u041a\u043e\u043b\u0438\u0447\u0435\u0441\u0442\u0432\u043e \u0441\u0442\u0440\u0430\u043d\u0438\u0446:')[0])
                        photo = 'http://eksmo.ru' + book.cssselect('center img')[0].get('src')
                        descr = book.xpath('//div[@class="grid_12 omega"]/br/following-sibling::text()')[0].strip()
                        date_publishing = book.xpath('//strong[text()="%s"]/following-sibling::span/text()' % u'\u0414\u0430\u0442\u0430 \u0432\u044b\u0445\u043e\u0434\u0430:')[0]
                        bookToBase(full_name, author, isbn, None, series, forma, pages, None, photo, descr, date_publishing, None, site_id, source_url, 0, external_id, fbclient)
                    except:
                        report(source_url + '   <-- Тут обламався парсер\n')
                    time.sleep(2)
    report(datetime.now().strftime("%d.%m.%Y %X") + ' кінець парсингу eksmo.ru\n')

def processObreey(pageurl):
    fbclient = fb('book')
    site_id = 'store.obreey.com'
    report(datetime.now().strftime("%d.%m.%Y %X") + ' початок парсингу store.obreey.com\n')
    page = parse(pageurl).getroot()
    for link in [pageurl + '?page=' + str(i) for i in range(1, int(page.cssselect('div.pager a')[-2].text) +1)]:
        books = parse(link).getroot()
        for source_url in ['http://store.obreey.com' + i.get('href') for i in books.cssselect('a.image')]:
            external_id = int(source_url.split('/')[-1])
            if fbclient.selectSQL('select 1 from books where site_id = ? and external_id = ?', (site_id, external_id)) == []:
                try:
                    book = parse(source_url, HTMLParser(encoding="utf-8"))
                    full_name = book.xpath('//span[@class="current"]/text()')[0]
	            #print full_name
	            try:
	                author = book.xpath('//th[text()="%s"]/following-sibling::td/a/text()' % u'\u0410\u0432\u0442\u043e\u0440')[0]
	            except:
                        author = None
	            try:
                        isbn = book.xpath('//meta[@name="og:book:isbn "]/@content')[0]
                    except:
                        isbn = None
                    try:
                        publisher = book.xpath('//th[text()="%s"]/following-sibling::td/a/text()' % u'\u0412\u0438\u0434\u0430\u0432\u0435\u0446\u044c')[0]
                    except:
                        publisher = None
                    try:
                        pages = int(book.xpath('//th[text()="%s"]/following-sibling::td/text()' %
                                           u'\u041a\u0456\u043b\u044c\u043a\u0456\u0441\u0442\u044c \u0441\u0442\u043e\u0440\u0456\u043d\u043e\u043a')[0])
                    except:
                        pages = None
                    photo = 'http://store.obreey.com' + book.xpath('//div[@class="item-view-options"]/img/@src')[0]
                    try:
                        descr = book.xpath('//div[@class="description description-with-markup"]/p/text()')[0]
                    except:
                        descr = None
                    try:
                        year_publishing = int(book.xpath('//meta[@name="og:book:release_date"]/@content')[0])
                    except:
                        year_publishing = None
                    bookToBase(full_name, author, isbn, publisher, None, None, pages, None, photo, descr, None, year_publishing, site_id, source_url, 0, external_id, fbclient)
                except:
                    report(source_url + '   <-- Тут обламався парсер\n')
    report(datetime.now().strftime("%d.%m.%Y %X") + ' кінець парсингу store.obreey.com\n')

def processBohdan(pageurl):
    fbclient = fb('book')
    site_id = 'bohdan-books.com'
    report(datetime.now().strftime("%d.%m.%Y %X") + ' початок парсингу bohdan-books.com\n')
    page = parse(pageurl).getroot()
    for section in [i.get('href') for i in page.cssselect('div.cat-item a')]:
        now = 1
        while True:
            books = parse(section + 'page_' + str(now)).getroot()
            for source_url in books.xpath('//td[@width="199"]/a/@href'):
                external_id = int(source_url.split('/')[-2][5:].replace('_', ''))
                if fbclient.selectSQL('select 1 from books where site_id = ? and external_id = ?', (site_id, external_id)) == []:
                    try:
                        book = parse(source_url.replace('https', 'http')).getroot()
                        full_name = book.cssselect('h1')[0].text
	                #print full_name
	                try:
	                    author = book.cssselect('a.auth_book')[0].text.strip()
	                except:
                            author = None
                        isbn = book.xpath('//b[text()="ISBN"]/following-sibling::text()')[0][2:]
                        try:
                            series = book.cssselect('span.auth_book a')[1].text
                        except:
                            series = None
                        forma = book.xpath('//b[text()="%s"]/following-sibling::text()' % u'\u0424\u043e\u0440\u043c\u0430\u0442')[0][2:]
                        try:
                            pages = int(book.xpath('//b[text()="%s"]/following-sibling::text()' %
                                                   u'\u041a\u0456\u043b\u044c\u043a\u0456\u0441\u0442\u044c \u0441\u0442\u043e\u0440\u0456\u043d\u043e\u043a')[0][2:])
                        except:
                            pages = None
                        try:
                            cover = book.xpath('//b[text()="%s"]/following-sibling::text()' % u'\u041e\u0431\u043a\u043b\u0430\u0434\u0438\u043d\u043a\u0430')[0][2:]
                        except:
                            cover = None
                        photo = book.cssselect('div img.img')[0].get('src').replace('https', 'http')
                        try:
                            descr = book.cssselect('div#btab-0')[0].text.strip()
                        except:
                            descr = None
                        year_publishing = int(book.xpath('//b[text()="%s"]/following-sibling::text()' % u'\u0420\u0456\u043a \u0432\u0438\u0434\u0430\u043d\u043d\u044f')[0][2:])
                        bookToBase(full_name, author, isbn, None, series, forma, pages, cover, photo, descr, None, year_publishing, site_id, source_url, 0, external_id, fbclient)
                    except:
                        report(source_url + '   <-- Тут обламався парсер\n')
            now += 1
            last = books.xpath('//div[@align="center"]/a[@title]/text()')[-1]
            if len(last)>4 or now > int(last):
                break
    report(datetime.now().strftime("%d.%m.%Y %X") + ' кінець парсингу bohdan-books.com\n')

def processMandrivets(pageurl):
    fbclient = fb('book')
    site_id = 'mandrivets.com'
    report(datetime.now().strftime("%d.%m.%Y %X") + ' початок парсингу mandrivets.com\n')
    page = parse(pageurl).getroot()
    for section in [i.get('href') for i in page.cssselect('table.Menu tr td a')][3:]:
        page = parse(section).getroot()
        if page.xpath('//b[text()="1"]/following-sibling::a/text()') == []:
            last = 1
        else:
            last = int( [i for i in page.xpath('//b[text()="1"]/following-sibling::a/text()') if i.isdigit()] [-1] )
        for link in [section + '&s=' + str(i) +'1' for i in range(last) ]:
            books = parse(link).getroot()
            for source_url in books.xpath('//a[img[@src="http://www.mandrivets.com/images/next.gif"]]/@href'):
                external_id = int(source_url.split('=')[-1])
                if fbclient.selectSQL('select 1 from books where site_id = ? and external_id = ?', (site_id, external_id)) == []:
                    try:
                        book = parse(source_url).getroot()
                        full_name = book.cssselect('strong')[1].text
	                #print full_name, external_id
	                author = book.cssselect('strong a')[1].text.strip()
                        isbn = book.cssselect('td.text')[3].text[5:] 
                        pages = int(book.cssselect('td.text')[5].text[20:])
                        photo = book.xpath('//a[@tar3get="_soopaPop:width=390,height=270,screenX=400,screenY=100,left=500,top=100"]/img/@src')[0].replace('small_', '')
                        try:
                            descr = book.cssselect('td.text')[6].text.strip()
                        except:
                            descr = None
                        year_publishing = int(book.cssselect('td.text')[4].text[13:])
                        bookToBase(full_name, author, isbn, None, None, None, pages, None, photo, descr, None, year_publishing, site_id, source_url, 0, external_id, fbclient)
                    except:
                        report(source_url + '   <-- Тут обламався парсер\n')
    report(datetime.now().strftime("%d.%m.%Y %X") + ' кінець парсингу mandrivets.com\n')

def processPublish(pageurl):
    fbclient = fb('book')
    site_id = 'publish.knig.net'
    report(datetime.now().strftime("%d.%m.%Y %X") + ' початок парсингу publish.knig.net\n')
    page = parse(pageurl).getroot()
    last = int(page.xpath('//td[@style="cursor:hand;cursor:pointer;color:#000000;font-weight:bold;"]/text()')[0][5:])
    for link in ['http://publish.knig.net/index.php?id=2&item_from_books=' + str(i) +'0' for i in range(last) ]:
        books = parse(link).getroot()
        for onebook in books.xpath('//td[@style="padding-left:26px;"]/a/@href'):
            source_url = 'http://publish.knig.net/' + onebook
            external_id = int(source_url.split('=')[-1])
            if fbclient.selectSQL('select 1 from books where site_id = ? and external_id = ?', (site_id, external_id)) == []:
                try:
                    book = parse(source_url).getroot()
                    full_name = book.cssselect('strong')[1].text.strip()
	            #print full_name, external_id
	            try:
	                author = book.xpath('//a[@title="%s"]/text()' %
                                            u'\u0412\u0441\u0435 \u043a\u043d\u0438\u0433\u0438 \u044d\u0442\u043e\u0433\u043e \u0430\u0432\u0442\u043e\u0440\u0430')[0]
	            except:
                        author = None
                    isbn = book.xpath('//td[strong[text()="ISBN:"]]/following-sibling::td/text()')[0]
                    publisher = book.xpath('//a[@title="%s"]/text()' %
                                           u'\u0412\u0441\u0435 \u043a\u043d\u0438\u0433\u0438 \u044d\u0442\u043e\u0433\u043e \u0438\u0437\u0434\u0430\u0442\u0435\u043b\u044c\u0441\u0442\u0432\u0430')[0]
                    try:
                        series = book.xpath('//a[@title="%s"]/text()' % u'\u0412\u0441\u0435 \u043a\u043d\u0438\u0433\u0438 \u044d\u0442\u043e\u0439 \u0441\u0435\u0440\u0438\u0438')[0]
                    except:
                        series = None
                    try:
                        forma = book.xpath('//a[@title="%s"]/text()' %
                                           u'\u0412\u0441\u0435 \u043a\u043d\u0438\u0433\u0438 \u044d\u0442\u043e\u0433\u043e \u0444\u043e\u0440\u043c\u0430\u0442\u0430')[0]
                    except:
                        forma = None
                    try:
                        pages = int(book.xpath('//td[@style="padding-left:5px;"]/text()')[0])
                    except:
                        pages = None
                    cover = book.xpath('//a[@title="%s"]/text()' %
                                       u'\u0412\u0441\u0435 \u043a\u043d\u0438\u0433\u0438 \u044d\u0442\u043e\u0433\u043e \u043f\u0435\u0440\u0435\u043f\u043b\u0435\u0442\u0430')[0]
                    photo = 'http://publish.knig.net' + book.xpath('//td[@style="padding-left:20px;padding-right:20px;"]/img/@src')[0][1:]
                    try:
                        descr = book.xpath('//tr[td[@style="padding-left:5px;padding-bottom:10px;"]]/following-sibling::tr/td//text()')[0].strip()
                    except:
                        descr = None
                    year_publishing = int(book.xpath('//td[strong[text()="%s"]]/following-sibling::td/text()' % u'\u0413\u043e\u0434 \u0438\u0437\u0434\u0430\u043d\u0438\u044f:')[0])
                    bookToBase(full_name, author, isbn, publisher, series, forma, pages, cover, photo, descr, None, year_publishing, site_id, source_url, 0, external_id, fbclient)
                except:
                    report(source_url + '   <-- Тут обламався парсер\n')
    report(datetime.now().strftime("%d.%m.%Y %X") + ' кінець парсингу publish.knig.net\n')

def processMachaon(pageurl):
    fbclient = fb('book')
    site_id = 'shop.machaon.kiev.ua'
    links = []
    report(datetime.now().strftime("%d.%m.%Y %X") + ' початок парсингу shop.machaon.kiev.ua\n')
    page = parse(pageurl).getroot()
    for section in ['http://shop.machaon.kiev.ua/' + i.get('href') for i in  page.cssselect('a.subcat_title')]:
        page = parse(section).getroot()
        if page.cssselect('a.cat_title') == []:
            links.append(section)
        else:
            for i in page.cssselect('td')[2:]:
	        if i.cssselect('a.subcat_title') == []:
		    links.append('http://shop.machaon.kiev.ua/' + i.cssselect('a.cat_title')[0].get('href'))
	        else:
		    [links.append('http://shop.machaon.kiev.ua/'+ j.get('href')) for j in i.cssselect('a.subcat_title')]
    for link in links:
        page = parse(link).getroot()
        npages = [link]
        [npages.append('http://shop.machaon.kiev.ua/'+ i) for i in page.xpath('//p[@align="left"][1]/a/@href') if page.cssselect('a.page') != []]
        for link in npages:
            books = parse(link).getroot()
            for onebook in books.cssselect('a.prod_name'):
                source_url = 'http://shop.machaon.kiev.ua/' + onebook.get('href')
                external_id = int(source_url.split('/')[-1][2:])
                if fbclient.selectSQL('select 1 from books where site_id = ? and external_id = ?', (site_id, external_id)) == []:
                    try:
                        book = parse(source_url).getroot()
                        full_name = book.cssselect('h1')[0].text.strip()
	                print full_name, external_id
	                author = book.cssselect('span.prod_btxt')[0].text.strip()
	                publisher = book.cssselect('span.prod_btxt')[1].text
	                try:
                            isbn = book.cssselect('span.prod_btxt')[3].text
	                    forma = book.cssselect('span.prod_btxt')[6].text
                            pages = int(book.cssselect('span.prod_btxt')[5].text.split()[0])
                            cover = book.cssselect('span.prod_btxt')[4].text
                        except:
                            isbn = book.cssselect('span.prod_btxt')[4].text
                            forma = book.cssselect('span.prod_btxt')[7].text
                            pages = int(book.cssselect('span.prod_btxt')[6].text.split()[0])
                            cover = book.cssselect('span.prod_btxt')[5].text
                        photo = 'http://shop.machaon.kiev.ua/' + book.xpath('//div[table]/a/@href')[0]
                        try:
                            descr = book.xpath('//table[2]/following-sibling::text()')[0].strip()
                        except:
                            descr = None
                        bookToBase(full_name, author, isbn, publisher, None, forma, pages, cover, photo, descr, None, None, site_id, source_url, 0, external_id, fbclient)
                    except:
                        report(source_url + '   <-- Тут обламався парсер\n')
    report(datetime.now().strftime("%d.%m.%Y %X") + ' кінець парсингу shop.machaon.kiev.ua\n')

def processMybook(pageurl):
    fbclient = fb('book')
    site_id = 'mybook.in.ua'
    report(datetime.now().strftime("%d.%m.%Y %X") + ' початок парсингу mybook.in.ua\n')
    page = parse(pageurl).getroot()
    last = int(page.cssselect('a.page')[-2].text)
    for link in ['http://mybook.in.ua/?mode=book&page=' + str(i) for i in range(1, last+1) ]:
        books = parse(link).getroot()
        for onebook in books.xpath('//a[@style="font-size:12px;"]/@href'):
            source_url = pageurl + onebook
            external_id = int(source_url[34:])
            if fbclient.selectSQL('select 1 from books where site_id = ? and external_id = ?', (site_id, external_id)) == []:
                try:
                    book = parse(source_url).getroot()
                    full_name = book.cssselect('h1')[0].text.strip()
	            #print full_name, external_id
	            author = ', '.join(book.xpath('//div[@class="book_author"]/a/text()'))
                    cover = book.xpath('//div[@style="line-height:16px;"]/br/following-sibling::text()')[0].strip()
                    photo = pageurl + book.xpath('//a[@rel="fancy_group_main"]/@href')[0]
                    descr = book.cssselect('div.book_text')[0].text
                    try:
                        isbn = book.xpath('//div[@style="line-height:16px;"]/br/following-sibling::text()')[4].strip()
                        forma = book.xpath('//div[@style="line-height:16px;"]/br/following-sibling::text()')[1].strip()
                        pages = int(book.xpath('//div[@style="line-height:16px;"]/br/following-sibling::text()')[3].strip()[17:])
                        year_publishing = int(book.xpath('//div[@style="line-height:16px;"]/br/following-sibling::text()')[2].strip()[14:])
                    except:
                        isbn = book.xpath('//div[@style="line-height:16px;"]/br/following-sibling::text()')[3].strip()
                        if isbn[10] == u'\u2116':
                            isbn = None
                        forma = None
                        pages = int(book.xpath('//div[@style="line-height:16px;"]/br/following-sibling::text()')[2].strip()[17:])
                        year_publishing = int(book.xpath('//div[@style="line-height:16px;"]/br/following-sibling::text()')[1].strip()[14:])
                    bookToBase(full_name, author, isbn, None, None, forma, pages, cover, photo, descr, None, year_publishing, site_id, source_url, 0, external_id, fbclient)
                except:
                    report(source_url + '   <-- Тут обламався парсер\n')
    report(datetime.now().strftime("%d.%m.%Y %X") + ' кінець парсингу mybook.in.ua\n')

def processPower(pageurl):
    fbclient = fb('book')
    site_id = 'power-book.com.ua'
    report(datetime.now().strftime("%d.%m.%Y %X") + ' початок парсингу mybook.in.ua\n')
    page = parse(pageurl).getroot()
    last = int(page.cssselect('div.nav-pages a')[-1].get('href').split("=")[-1])
    for link in ['http://www.power-book.com.ua/e-store/books/?PAGEN_1=' + str(i) for i in range(1, last+1) ]:
        books = parse(link).getroot()
        for onebook in books.xpath('//td[@style="min-width: 110px"]/a/@href'):
            source_url = 'http://www.power-book.com.ua' + onebook
            external_id = int(source_url.split("=")[1])
            if fbclient.selectSQL('select 1 from books where site_id = ? and external_id = ?', (site_id, external_id)) == []:
                try:
                    book = parse(source_url).getroot()
                    full_name = book.cssselect('h1')[0].text.strip()
	            #print full_name, external_id
	            try:
	                author = book.xpath('//span[text()="%s"]/following-sibling::b/a/text()' % u'\u0410\u0432\u0442\u043e\u0440(\u044b)')[0]
	            except:
                        author = None
                    try:
	                isbn = book.xpath('//span[text()="ISBN"]/following-sibling::b/text()')[0].strip()
	            except:
                        isbn = None
                    try:
	                publisher = book.xpath('//span[text()="%s"]/following-sibling::b/a/text()' % u'\u0418\u0437\u0434\u0430\u0442\u0435\u043b\u044c\u0441\u0442\u0432\u043e')[0]
	            except:
                        publisher = None
	            try:
	                series = book.xpath('//span[text()="%s"]/following-sibling::b/text()' % u'\u0421\u0435\u0440\u0438\u044f')[0].strip()
	            except:
                        series = None
                    try:
                        forma = book.xpath('//span[text()="%s"]/following-sibling::b/text()' % u'\u0424\u043e\u0440\u043c\u0430\u0442')[0].strip()
                    except:
                        forma = None
                    try:
                        pages = int(book.xpath('//span[text()="%s"]/following-sibling::b/text()' % u'\u0427\u0438\u0441\u043b\u043e \u0441\u0442\u0440\u0430\u043d\u0438\u0446')[0].strip())
                    except:
                        pages = None
                    try:    
                        cover = book.xpath('//span[text()="%s"]/following-sibling::b/text()' % u'\u0422\u0432\u0435\u0440\u0434\u0430\u044f/\u041c\u044f\u0433\u043a\u0430\u044f')[0].strip()
                    except:
                        cover = None
                    photo = book.cssselect('img#catalog_detail_image')[0].get('src')
                    if photo[: 4] != 'http':
                        photo = 'http://www.power-book.com.ua' + photo
                    try:
                        descr = book.xpath('//div[@class="catalog-detail-full-desc"]/text()')[2].strip()
                    except:
                        descr = None
                    try:
                        year_publishing = int(book.cssselect('b')[0].text.strip())
                    except:
                        year_publishing = None
                    bookToBase(full_name, author, isbn, publisher, series, forma, pages, cover, photo, descr, None, year_publishing, site_id, source_url, 0, external_id, fbclient)
                except:
                    report(source_url + '   <-- Тут обламався парсер\n')
    report(datetime.now().strftime("%d.%m.%Y %X") + ' кінець парсингу power-book.com.ua\n')

def processMoipokupki(pageurl):
    fbclient = fb('book')
    site_id = 'moipokupki.com.ua'
    report(datetime.now().strftime("%d.%m.%Y %X") + ' початок парсингу moipokupki.com.ua\n')
    page = parse(pageurl).getroot()
    for section in ['http://moipokupki.com.ua' + i for i in page.xpath('//li[@class="parent_item"]/div/a/@href')]:
        now = 1
        while True:
            books = parse(section + '?PAGEN_1=' + str(now)).getroot()
            for onebook in books.cssselect('div.article_title a'):
                source_url = 'http://moipokupki.com.ua' + onebook.get('href')
                if fbclient.selectSQL('select 1 from books where source_url = ?', (source_url,)) == []:
                    try:
                        book = parse(source_url).getroot()
                        #print source_url
                        full_name = book.cssselect('h1')[0].text
                        try:
	                    author = book.xpath('//span[text()="%s"]/following-sibling::div/text()' % u'\u0410\u0432\u0442\u043e\u0440')[0].strip()
	                except:
                            author = None
	                isbn = book.xpath('//span[text()="ISBN"]/following-sibling::div/text()')[0].strip()
	                publisher = book.xpath('//span[text()="%s"]/following-sibling::div/text()' % u'\u0418\u0437\u0434\u0430\u0442\u0435\u043b\u044c\u0441\u0442\u0432\u043e')[0].strip()
	                series = book.xpath('//span[text()="%s"]/following-sibling::div/text()' % u'\u0421\u0435\u0440\u0438\u044f')[0].strip()
	                try:
	                    forma = book.xpath('//span[text()="%s"]/following-sibling::div/text()' %
                                               u'\u0413\u0430\u0431\u0430\u0440\u0438\u0442\u044b (\u0414 \u0445 \u0428 \u0445 \u0412)')[0].strip()
	                except:
                            forma = None
                        try:
                            pages = int(book.xpath('//span[text()="%s"]/following-sibling::div/text()' %
                                                   u'\u041a\u043e\u043b\u0438\u0447\u0435\u0441\u0442\u0432\u043e \u0441\u0442\u0440\u0430\u043d\u0438\u0446')[0].strip())
                        except:
                            pages = None
                        try:
                            cover = book.xpath('//span[text()="%s"]/following-sibling::div/text()' % u'\u041f\u0435\u0440\u0435\u043f\u043b\u0435\u0442')[0].strip()
                        except:
                            cover = None
                        photo = 'http://moipokupki.com.ua' + book.cssselect('a.fancy_images')[0].get('href')
                        try:
                            descr = book.xpath('//div[@id="tabs-1"]/text()')[1].strip()
                            if descr == '':
                                descr = book.xpath('//div[@id="tabs-1"]/text()')[2].strip()
                        except:
                            descr = None
                        try:
                            year_publishing = int(book.xpath('//span[text()="%s"]/following-sibling::div/text()' %
                                                             u'\u0413\u043e\u0434 \u0438\u0437\u0434\u0430\u043d\u0438\u044f')[0].strip())
                        except:
                            year_publishing = None
                        bookToBase(full_name, author, isbn, publisher, series, forma, pages, cover, photo, descr, None, year_publishing, site_id, source_url, 0, None, fbclient)
                    except:
                        report(source_url + '   <-- Тут обламався парсер\n')
            now += 1
            if  now > int(books.cssselect('a.nav_item')[-2].text):
                break
    report(datetime.now().strftime("%d.%m.%Y %X") + ' кінець парсингу moipokupki.com.ua\n')
    
def yarnToBase(manufacturer, series, content, length, weight, using, photo, site_id, source_url, parsing_result, external_id, color_group, fbclient):
    qu = fbclient.selectSQL('''insert into yarns(manufacturer, series, contents, length, weight, use_method, photo, site_id, source_url, parsing_result, external_id)
                                                    values(?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?) returning id''', 
                            (manufacturer, series, content, length, weight, using, urllib.urlopen(photo), site_id, source_url, 0, external_id))
    if qu[0] == -1:
        report('\n' + 'Error processing category %s' % series)
    #print 'Adding colors for %s' % series
    yarn_id = qu[0]
    for colorcode, color_img in color_group:
        qu = fbclient.execSQL('insert into yarncolors(yarn_id,color_id,photo)values(?,?,?)', (yarn_id[0], colorcode, urllib.urlopen(color_img)))
        if qu[0] != 0:
            report('\n' + "%s happened during executing %s" %(qu[1], qu[2]))

def bookToBase(full_name, author, isbn, publisher, series, forma, pages, cover, photo, descr, date_publishing,
               year_publishing, site_id, source_url, parsing_result, external_id, fbclient):
    qu = fbclient.execSQL('''insert into books(full_name, author, isbn, publisher, series, format, pages, cover, photo, descr, date_publishing,
                                year_publishing, site_id, source_url, parsing_result, external_id) values(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)''',
                            (full_name, author, isbn, publisher, series, forma, pages, cover, urllib.urlopen(photo),
                                descr, date_publishing, year_publishing, site_id, source_url, parsing_result, external_id))
    if qu[0] != 0:
        qu = fbclient.execSQL('insert into books(source_url, parsing_result) values(?, ?)', (source_url, -1))
        report(source_url + '   <-- Цього не хоче база\n')
       
def report(line):
    lock.acquire()
    rfile.write(line)
    lock.release()
          
if __name__ == '__main__':
    fb_config = ConfigObj(os.path.join(os.path.dirname(__file__), 'firebird.ini'))
    kinterbasdb.init(concurrency_level=2)
    lock = Lock()
    p1 = Thread(target=processYarnart, args=['http://www.yarnart.info/Urunlerimiz.aspx',])
    p2 = Thread(target=processAlize, args=['http://www.alize.gen.tr/index_en.php?is=hand_knitting_yarn_collection&sayfa=vitrin2',])
    p3 = Thread(target=processKartopu, args=['http://www.kartopu.com.tr/EN/Products/',])
    p4 = Thread(target=processGazzal, args=['http://www.gazzal.net/?deger=sayfa/YARNS&secim=3&dil=1',])
    p5 = Thread(target=processVizell, args=['http://www.vizell.com.ua/vizell/vizell',])
    p6 = Thread(target=processPeriayarn, args=['http://www.periayarn.com.ua/products.php?pid=23',])
    p7 = Thread(target=processZnannia, args=['http://www.znannia.com.ua/index.php?page=shop.browse&category_id=6&option=com_virtuemart&Itemid=2',])
    p8 = Thread(target=processOzon, args=['http://www.ozon.ru/context/detail/id/',])
    p9 = Thread(target=processKnigka, args=['http://knigka.ua/',])
    p10 = Thread(target=processBao, args=['http://www.bao-book.com/books/show/id/',])
    p11 = Thread(target=processSchoolbook, args=['http://www.schoolbook.com.ua//catalog.php',])
    p12 = Thread(target=processEksmo, args=['http://eksmo.ru/catalog/',])
    p14 = Thread(target=processObreey, args=['http://store.obreey.com/ukr/books',])
    p15 = Thread(target=processBohdan, args=['http://www.bohdan-books.com/',])
    p16 = Thread(target=processMandrivets, args=['http://mandrivets.com/',])
    p17 = Thread(target=processPublish, args=['http://publish.knig.net/index.php?id=2&bfilter_on=0',])
    p18 = Thread(target=processMachaon, args=['http://shop.machaon.kiev.ua/eshop',])
    p19 = Thread(target=processMybook, args=['http://mybook.in.ua/',])
    p20 = Thread(target=processPower, args=['http://www.power-book.com.ua/e-store/books/',])
    p21 = Thread(target=processMoipokupki, args=['http://moipokupki.com.ua/sitemap/',])
    p_dict = {'yarnart.info':p1, 'alize.gen.tr':p2, 'kartopu.com.tr':p3, 'gazzal.net':p4, 'vizell.com.ua':p5, 'periayarn.com.ua':p6, 'znannia.com.ua':p7, 'ozon.ru':p8, 'knigka.ua':p9,
             'bao-book.com':p10, 'schoolbook.com.ua':p11, 'eksmo.ru':p12, 'store.obreey.com':p14, 'bohdan-books.com':p15, 'mandrivets.com':p16, 'publish.knig.net':p17,
             'shop.machaon.kiev.ua':p18, 'mybook.in.ua':p19, 'power-book.com.ua':p20, 'moipokupki.com.ua':p21}
    parser = argparse.ArgumentParser()
    parser.add_argument('-s', '--site', dest='site', action='append', help='наприклад: -s ozon.ru -s store.obreey.com')
    parser.add_argument('-t', '--type', dest='category', help='book, yarn або all')

    pathTolog = fb_config['log']['name']
    try:
        rfile = open(pathTolog, 'a', 0)
        if os.statvfs(pathTolog).f_bavail < 2560:
             report('На диску мало вільного місця\n')
    except IOError:
        print 'Помилка вводу-виводу: ', pathTolog
        exit()

    category = parser.parse_args().category
    if category != None:
        if category == 'yarn':
            p_category = [p1, p2, p3, p4, p5, p6]
        if category == 'book':
            p_category = [p7, p8, p9, p10, p11, p12, p14, p15, p16, p17, p18, p19, p20, p21]
        if category == 'all':
            p_category = [p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p14, p15, p16, p17, p18, p19, p20, p21]
        for i in p_category:
            i.start()
        for i in p_category:
            i.join()

    sites = parser.parse_args().site
    if sites != None:
        for i in sites:
            p_dict.get(i).start()
        for i in sites:
            p_dict.get(i).join()

    rfile.close()
